def build_network_list() -> None:
    """use https://github.com/ethereum-lists/chains repo to construct list"""
    raise NotImplementedError()


def add_network_to_disk():
    raise NotImplementedError()

