"""WIP

Desired Information
- token creation date
- holders
    - top token holders
    - number of holders
- token dex summary on major exchanges
    - volume
    - liquidity and liquidity depth
- token CEX activity and inflow/outflow
- how each of these values change over time
"""

