from ctc import spec


def digest_shh_version(
    response: spec.RpcSingularResponse,
) -> spec.RpcSingularResponse:
    return response


def digest_shh_post(
    response: spec.RpcSingularResponse,
) -> spec.RpcSingularResponse:
    return response


def digest_shh_new_identity(
    response: spec.RpcSingularResponse,
) -> spec.RpcSingularResponse:
    return response


def digest_shh_has_identity(
    response: spec.RpcSingularResponse,
) -> spec.RpcSingularResponse:
    return response


def digest_shh_new_group(
    response: spec.RpcSingularResponse,
) -> spec.RpcSingularResponse:
    return response


def digest_shh_add_to_group(
    response: spec.RpcSingularResponse,
) -> spec.RpcSingularResponse:
    return response


def digest_shh_new_filter(
    response: spec.RpcSingularResponse,
) -> spec.RpcSingularResponse:
    return response


def digest_shh_uninstall_filter(
    response: spec.RpcSingularResponse,
) -> spec.RpcSingularResponse:
    return response


def digest_shh_get_filter_changes(
    response: spec.RpcSingularResponse,
) -> spec.RpcSingularResponse:
    return response


def digest_shh_get_messages(
    response: spec.RpcSingularResponse,
) -> spec.RpcSingularResponse:
    return response

