from ctc import spec


def digest_eth_get_compilers(
    response: spec.RpcSingularResponse,
) -> spec.RpcSingularResponse:
    return response


def digest_eth_compile_lll(
    response: spec.RpcSingularResponse,
) -> spec.RpcSingularResponse:
    return response


def digest_eth_compile_solidity(
    response: spec.RpcSingularResponse,
) -> spec.RpcSingularResponse:
    return response


def digest_eth_compile_serpent(
    response: spec.RpcSingularResponse,
) -> spec.RpcSingularResponse:
    return response

